@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -22,16 +22,21 @@ import os
 import socket
 from httplib import HTTPException
 from time import time
-from urllib import quote as _quote
 
 from eventlet import sleep, Timeout
 
 from swift.common.bufferedhttp import http_connect
-from swiftclient import ClientException, json_loads
+from swift.common.exceptions import ClientException
 from swift.common.utils import normalize_timestamp, FileLikeIter
 from swift.common.http import HTTP_NO_CONTENT, HTTP_INSUFFICIENT_STORAGE, \
     is_success, is_server_error
 from swift.common.swob import HeaderKeyDict
+from swift.common.utils import quote
+
+try:
+    import simplejson as json
+except ImportError:
+    import json
 
 
 def _get_direct_account_container(path, stype, node, part,
@@ -74,13 +79,7 @@ def _get_direct_account_container(path, stype, node, part,
     if resp.status == HTTP_NO_CONTENT:
         resp.read()
         return resp_headers, []
-    return resp_headers, json_loads(resp.read())
-
-
-def quote(value, safe='/'):
-    if isinstance(value, unicode):
-        value = value.encode('utf8')
-    return _quote(value, safe)
+    return resp_headers, json.loads(resp.read())
 
 
 def gen_headers(hdrs_in=None, add_ts=False):
@@ -475,12 +474,12 @@ def retry(func, *args, **kwargs):
         attempts += 1
         try:
             return attempts, func(*args, **kwargs)
-        except (socket.error, HTTPException, Timeout), err:
+        except (socket.error, HTTPException, Timeout) as err:
             if error_log:
                 error_log(err)
             if attempts > retries:
                 raise
-        except ClientException, err:
+        except ClientException as err:
             if error_log:
                 error_log(err)
             if attempts > retries or not is_server_error(err.http_status) or \
