@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,8 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from gettext import gettext as _
-from eventlet import Timeout
+from swift import gettext_ as _
 
 from swift.common.swob import Request, HTTPServerError
 from swift.common.utils import get_logger, generate_trans_id
@@ -35,8 +34,8 @@ class CatchErrorsContext(WSGIContext):
         try:
             # catch any errors in the pipeline
             resp = self._app_call(env)
-        except (Exception, Timeout), err:
-            self.logger.exception(_('Error: %s'), err)
+        except:  # noqa
+            self.logger.exception(_('Error: An error occurred'))
             resp = HTTPServerError(request=Request(env),
                                    body='An error occurred',
                                    content_type='text/plain')
