@@ -1,5 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-# Copyright 2012 OpenStack LLC
+# Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
@@ -16,6 +15,7 @@
 from swift.common import utils as swift_utils
 from swift.common.middleware import acl as swift_acl
 from swift.common.swob import HTTPNotFound, HTTPForbidden, HTTPUnauthorized
+from swift.common.utils import register_swift_info
 
 
 class KeystoneAuth(object):
@@ -38,9 +38,9 @@ class KeystoneAuth(object):
     installing keystone.
 
     If support is required for unvalidated users (as with anonymous
-    access) or for tempurl/formpost middleware, authtoken will need
-    to be configured with ``delay_auth_decision`` set to 1.  See the
-    Keystone documentation for more detail on how to configure the
+    access) or for formpost/staticweb/tempurl middleware, authtoken will
+    need to be configured with ``delay_auth_decision`` set to true.  See
+    the Keystone documentation for more detail on how to configure the
     authtoken middleware.
 
     In proxy-server.conf you will need to have the setting account
@@ -100,7 +100,7 @@ class KeystoneAuth(object):
             return self.app(environ, start_response)
 
         if identity:
-            self.logger.debug('Using identity: %r' % (identity))
+            self.logger.debug('Using identity: %r', identity)
             environ['keystone.identity'] = identity
             environ['REMOTE_USER'] = identity.get('tenant')
             environ['swift.authorize'] = self.authorize
@@ -200,7 +200,7 @@ class KeystoneAuth(object):
         # role.
         if self.reseller_admin_role in user_roles:
             msg = 'User %s has reseller admin authorizing'
-            self.logger.debug(msg % tenant_id)
+            self.logger.debug(msg, tenant_id)
             req.environ['swift_owner'] = True
             return
 
@@ -209,7 +209,7 @@ class KeystoneAuth(object):
         if not container and not obj and req.method == 'DELETE':
             # User is not allowed to issue a DELETE on its own account
             msg = 'User %s:%s is not allowed to delete its own account'
-            self.logger.debug(msg % (tenant_name, user_name))
+            self.logger.debug(msg, tenant_name, user_name)
             return self.denied_response(req)
 
         # cross-tenant authorization
@@ -217,8 +217,8 @@ class KeystoneAuth(object):
                                                    tenant_id, tenant_name,
                                                    roles)
         if matched_acl is not None:
-            log_msg = 'user %s allowed in ACL authorizing.' % matched_acl
-            self.logger.debug(log_msg)
+            log_msg = 'user %s allowed in ACL authorizing.'
+            self.logger.debug(log_msg, matched_acl)
             return
 
         acl_authorized = self._authorize_unconfirmed_identity(req, obj,
@@ -230,8 +230,8 @@ class KeystoneAuth(object):
         # Check if a user tries to access an account that does not match their
         # token
         if not self._reseller_check(account, tenant_id):
-            log_msg = 'tenant mismatch: %s != %s' % (account, tenant_id)
-            self.logger.debug(log_msg)
+            log_msg = 'tenant mismatch: %s != %s'
+            self.logger.debug(log_msg, account, tenant_id)
             return self.denied_response(req)
 
         # Check the roles the user is belonging to. If the user is
@@ -241,8 +241,8 @@ class KeystoneAuth(object):
         for role in self.operator_roles.split(','):
             role = role.strip()
             if role in user_roles:
-                log_msg = 'allow user with role %s as account admin' % (role)
-                self.logger.debug(log_msg)
+                log_msg = 'allow user with role %s as account admin'
+                self.logger.debug(log_msg, role)
                 req.environ['swift_owner'] = True
                 return
 
@@ -261,8 +261,8 @@ class KeystoneAuth(object):
         for user_role in user_roles:
             if user_role in (r.lower() for r in roles):
                 log_msg = 'user %s:%s allowed in ACL: %s authorizing'
-                self.logger.debug(log_msg % (tenant_name, user_name,
-                                             user_role))
+                self.logger.debug(log_msg, tenant_name, user_name,
+                                  user_role)
                 return
 
         return self.denied_response(req)
@@ -307,15 +307,15 @@ class KeystoneAuth(object):
                 and (req.environ['swift_sync_key'] ==
                      req.headers.get('x-container-sync-key', None))
                 and 'x-timestamp' in req.headers):
-            log_msg = 'allowing proxy %s for container-sync' % req.remote_addr
-            self.logger.debug(log_msg)
+            log_msg = 'allowing proxy %s for container-sync'
+            self.logger.debug(log_msg, req.remote_addr)
             return True
 
         # Check if referrer is allowed.
         if swift_acl.referrer_allowed(req.referer, referrers):
             if obj or '.rlistings' in roles:
-                log_msg = 'authorizing %s via referer ACL' % req.referrer
-                self.logger.debug(log_msg)
+                log_msg = 'authorizing %s via referer ACL'
+                self.logger.debug(log_msg, req.referrer)
                 return True
             return False
 
@@ -335,6 +335,7 @@ def filter_factory(global_conf, **local_conf):
     """Returns a WSGI filter app for use with paste.deploy."""
     conf = global_conf.copy()
     conf.update(local_conf)
+    register_swift_info('keystoneauth')
 
     def auth_filter(app):
         return KeystoneAuth(app, conf)
