@@ -1,4 +1,4 @@
-# Copyright (c) 2012 OpenStack, LLC.
+# Copyright (c) 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,18 +15,22 @@
 '''
 Created on February 27, 2012
 
-A filter that disallows any paths that contain defined forbidden characters
-or that exceed a defined length.
+A filter that disallows any paths that contain defined forbidden characters or
+that exceed a defined length.
 
-Place in proxy filter before proxy, e.g.
+Place early in the proxy-server pipeline after the left-most occurrence of the
+``proxy-logging`` middleware (if present) and before the final
+``proxy-logging`` middleware (if present) or the ``proxy-serer`` app itself,
+e.g.::
 
-[pipeline:main]
-pipeline = catch_errors healthcheck name_check cache tempauth sos proxy-server
+    [pipeline:main]
+    pipeline = catch_errors healthcheck proxy-logging name_check cache \
+ratelimit tempauth sos proxy-logging proxy-server
 
-[filter:name_check]
-use = egg:swift#name_check
-forbidden_chars = '"`<>
-maximum_length = 255
+    [filter:name_check]
+    use = egg:swift#name_check
+    forbidden_chars = '"`<>
+    maximum_length = 255
 
 There are default settings for forbidden_chars (FORBIDDEN_CHARS) and
 maximum_length (MAX_LENGTH)
