@@ -28,6 +28,7 @@ import threading as stdlib_threading
 import time
 import uuid
 import functools
+import weakref
 from hashlib import md5, sha1
 from random import random, shuffle
 from urllib import quote as _quote
@@ -710,7 +711,7 @@ class GreenthreadSafeIterator(object):
             return self.unsafe_iter.next()
 
 
-class NullLogger():
+class NullLogger(object):
     """A no-op logger for eventlet wsgi."""
 
     def write(self, *args):
@@ -848,6 +849,26 @@ def timing_stats(**dec_kwargs):
     return decorating_func
 
 
+class LoggingHandlerWeakRef(weakref.ref):
+    """
+    Like a weak reference, but passes through a couple methods that logging
+    handlers need.
+    """
+    def close(self):
+        referent = self()
+        try:
+            if referent:
+                referent.close()
+        except KeyError:
+            # This is to catch an issue with old py2.6 versions
+            pass
+
+    def flush(self):
+        referent = self()
+        if referent:
+            referent.flush()
+
+
 # double inheritance to support property with setter
 class LogAdapter(logging.LoggerAdapter, object):
     """
@@ -1131,6 +1152,32 @@ def get_logger(conf, name=None, log_to_console=False, log_route=None,
                 print >>sys.stderr, 'Error calling custom handler [%s]' % hook
             except ValueError:
                 print >>sys.stderr, 'Invalid custom handler format [%s]' % hook
+
+    # Python 2.6 has the undesirable property of keeping references to all log
+    # handlers around forever in logging._handlers and logging._handlerList.
+    # Combine that with handlers that keep file descriptors, and you get an fd
+    # leak.
+    #
+    # And no, we can't share handlers; a SyslogHandler has a socket, and if
+    # two greenthreads end up logging at the same time, you could get message
+    # overlap that garbles the logs and makes eventlet complain.
+    #
+    # Python 2.7 uses weakrefs to avoid the leak, so let's do that too.
+    if sys.version_info[0] == 2 and sys.version_info[1] <= 6:
+        try:
+            logging._acquireLock()  # some thread-safety thing
+            for handler in adapted_logger.logger.handlers:
+                if handler in logging._handlers:
+                    wr = LoggingHandlerWeakRef(handler)
+                    del logging._handlers[handler]
+                    logging._handlers[wr] = 1
+                for i, handler_ref in enumerate(logging._handlerList):
+                    if handler_ref is handler:
+                        logging._handlerList[i] = LoggingHandlerWeakRef(
+                            handler)
+        finally:
+            logging._releaseLock()
+
     return adapted_logger
 
 
@@ -1711,26 +1758,41 @@ def ratelimit_sleep(running_time, max_rate, incr_by=1, rate_buffer=5):
     as eventlet.sleep() does involve some overhead.  Returns running_time
     that should be used for subsequent calls.
 
-    :param running_time: the running time of the next allowable request. Best
-                         to start at zero.
+    :param running_time: the running time in milliseconds of the next
+                         allowable request. Best to start at zero.
     :param max_rate: The maximum rate per second allowed for the process.
     :param incr_by: How much to increment the counter.  Useful if you want
                     to ratelimit 1024 bytes/sec and have differing sizes
-                    of requests. Must be >= 0.
+                    of requests. Must be > 0 to engage rate-limiting
+                    behavior.
     :param rate_buffer: Number of seconds the rate counter can drop and be
                         allowed to catch up (at a faster than listed rate).
                         A larger number will result in larger spikes in rate
-                        but better average accuracy.
+                        but better average accuracy. Must be > 0 to engage
+                        rate-limiting behavior.
     '''
-    if not max_rate or incr_by <= 0:
+    if max_rate <= 0 or incr_by <= 0:
         return running_time
+
+    # 1,000 milliseconds = 1 second
     clock_accuracy = 1000.0
+
+    # Convert seconds to milliseconds
     now = time.time() * clock_accuracy
+
+    # Calculate time per request in milliseconds
     time_per_request = clock_accuracy * (float(incr_by) / max_rate)
+
+    # Convert rate_buffer to milliseconds and compare
     if now - running_time > rate_buffer * clock_accuracy:
         running_time = now
     elif running_time - now > time_per_request:
+        # Convert diff back to a floating point number of seconds and sleep
         eventlet.sleep((running_time - now) / clock_accuracy)
+
+    # Return the absolute time for the next interval in milliseconds; note
+    # that time could have passed well beyond that point, but the next call
+    # will catch that and skip the sleep.
     return running_time + time_per_request
 
 
@@ -2036,6 +2098,28 @@ def human_readable(value):
     return '%d%si' % (round(value), suffixes[index])
 
 
+def put_recon_cache_entry(cache_entry, key, item):
+    """
+    Function that will check if item is a dict, and if so put it under
+    cache_entry[key].  We use nested recon cache entries when the object
+    auditor runs in 'once' mode with a specified subset of devices.
+    """
+    if isinstance(item, dict):
+        if key not in cache_entry or key in cache_entry and not \
+                isinstance(cache_entry[key], dict):
+            cache_entry[key] = {}
+        elif key in cache_entry and item == {}:
+            cache_entry.pop(key, None)
+            return
+        for k, v in item.items():
+            if v == {}:
+                cache_entry[key].pop(k, None)
+            else:
+                cache_entry[key][k] = v
+    else:
+        cache_entry[key] = item
+
+
 def dump_recon_cache(cache_dict, cache_file, logger, lock_timeout=2):
     """Update recon cache values
 
@@ -2055,7 +2139,7 @@ def dump_recon_cache(cache_dict, cache_file, logger, lock_timeout=2):
                 #file doesn't have a valid entry, we'll recreate it
                 pass
             for cache_key, cache_value in cache_dict.items():
-                cache_entry[cache_key] = cache_value
+                put_recon_cache_entry(cache_entry, cache_key, cache_value)
             try:
                 with NamedTemporaryFile(dir=os.path.dirname(cache_file),
                                         delete=False) as tf:
