@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,7 @@ import urllib
 from random import random
 from time import time
 from os.path import join
-from gettext import gettext as _
+from swift import gettext_ as _
 import hashlib
 
 from eventlet import sleep, Timeout
@@ -44,7 +44,7 @@ class ObjectExpirer(Daemon):
         self.interval = int(conf.get('interval') or 300)
         self.expiring_objects_account = \
             (conf.get('auto_create_account_prefix') or '.') + \
-            'expiring_objects'
+            (conf.get('expiring_objects_account_name') or 'expiring_objects')
         conf_path = conf.get('__file__') or '/etc/swift/object-expirer.conf'
         request_tries = int(conf.get('request_tries') or 3)
         self.swift = InternalClient(conf_path,
@@ -134,7 +134,7 @@ class ObjectExpirer(Daemon):
                         self.expiring_objects_account,
                         container,
                         acceptable_statuses=(2, HTTP_NOT_FOUND, HTTP_CONFLICT))
-                except (Exception, Timeout), err:
+                except (Exception, Timeout) as err:
                     self.logger.exception(
                         _('Exception while deleting container %s %s') %
                         (container, str(err)))
@@ -205,7 +205,7 @@ class ObjectExpirer(Daemon):
                                      container, obj)
             self.report_objects += 1
             self.logger.increment('objects')
-        except (Exception, Timeout), err:
+        except (Exception, Timeout) as err:
             self.logger.increment('errors')
             self.logger.exception(
                 _('Exception while deleting object %s %s %s') %
