@@ -35,7 +35,7 @@ from swift.common.constraints import check_object_creation, \
     check_float, check_utf8
 from swift.common.exceptions import ConnectionTimeout, DiskFileQuarantined, \
     DiskFileNotExist, DiskFileCollision, DiskFileNoSpace, DiskFileDeleted, \
-    DiskFileDeviceUnavailable, DiskFileExpired
+    DiskFileDeviceUnavailable, DiskFileExpired, ChunkReadTimeout
 from swift.obj import ssync_receiver
 from swift.common.http import is_success
 from swift.common.request_helpers import split_and_validate_path, is_user_meta
@@ -388,6 +388,16 @@ class ObjectController(object):
             orig_metadata = disk_file.read_metadata()
         except (DiskFileNotExist, DiskFileQuarantined):
             orig_metadata = {}
+
+        # Checks for If-None-Match
+        if request.if_none_match is not None and orig_metadata:
+            if '*' in request.if_none_match:
+                # File exists already so return 412
+                return HTTPPreconditionFailed(request=request)
+            if orig_metadata.get('ETag') in request.if_none_match:
+                # The current ETag matches, so return 412
+                return HTTPPreconditionFailed(request=request)
+
         orig_timestamp = orig_metadata.get('X-Timestamp')
         if orig_timestamp and orig_timestamp >= request.headers['x-timestamp']:
             return HTTPConflict(request=request)
@@ -398,15 +408,23 @@ class ObjectController(object):
         try:
             with disk_file.create(size=fsize) as writer:
                 upload_size = 0
-                reader = request.environ['wsgi.input'].read
-                for chunk in iter(lambda: reader(self.network_chunk_size), ''):
-                    start_time = time.time()
-                    if start_time > upload_expiration:
-                        self.logger.increment('PUT.timeouts')
-                        return HTTPRequestTimeout(request=request)
-                    etag.update(chunk)
-                    upload_size = writer.write(chunk)
-                    elapsed_time += time.time() - start_time
+
+                def timeout_reader():
+                    with ChunkReadTimeout(self.client_timeout):
+                        return request.environ['wsgi.input'].read(
+                            self.network_chunk_size)
+
+                try:
+                    for chunk in iter(lambda: timeout_reader(), ''):
+                        start_time = time.time()
+                        if start_time > upload_expiration:
+                            self.logger.increment('PUT.timeouts')
+                            return HTTPRequestTimeout(request=request)
+                        etag.update(chunk)
+                        upload_size = writer.write(chunk)
+                        elapsed_time += time.time() - start_time
+                except ChunkReadTimeout:
+                    return HTTPRequestTimeout(request=request)
                 if upload_size:
                     self.logger.transfer_rate(
                         'PUT.' + device + '.timing', elapsed_time,
@@ -473,21 +491,16 @@ class ObjectController(object):
                 obj_size = int(metadata['Content-Length'])
                 file_x_ts = metadata['X-Timestamp']
                 file_x_ts_flt = float(file_x_ts)
-                try:
-                    if_unmodified_since = request.if_unmodified_since
-                except (OverflowError, ValueError):
-                    # catches timestamps before the epoch
-                    return HTTPPreconditionFailed(request=request)
                 file_x_ts_utc = datetime.fromtimestamp(file_x_ts_flt, UTC)
+
+                if_unmodified_since = request.if_unmodified_since
                 if if_unmodified_since and file_x_ts_utc > if_unmodified_since:
                     return HTTPPreconditionFailed(request=request)
-                try:
-                    if_modified_since = request.if_modified_since
-                except (OverflowError, ValueError):
-                    # catches timestamps before the epoch
-                    return HTTPPreconditionFailed(request=request)
+
+                if_modified_since = request.if_modified_since
                 if if_modified_since and file_x_ts_utc <= if_modified_since:
                     return HTTPNotModified(request=request)
+
                 keep_cache = (self.keep_cache_private or
                               ('X-Auth-Token' not in request.headers and
                                'X-Storage-Token' not in request.headers))
