@@ -130,6 +130,8 @@ class Application(object):
             a.strip()
             for a in conf.get('cors_allow_origin', '').split(',')
             if a.strip()]
+        self.strict_cors_mode = config_true_value(
+            conf.get('strict_cors_mode', 't'))
         self.node_timings = {}
         self.timing_expiry = int(conf.get('timing_expiry', 300))
         self.sorting_method = conf.get('sorting_method', 'shuffle').lower()
@@ -210,7 +212,8 @@ class Application(object):
             container_listing_limit=constraints.CONTAINER_LISTING_LIMIT,
             max_account_name_length=constraints.MAX_ACCOUNT_NAME_LENGTH,
             max_container_name_length=constraints.MAX_CONTAINER_NAME_LENGTH,
-            max_object_name_length=constraints.MAX_OBJECT_NAME_LENGTH)
+            max_object_name_length=constraints.MAX_OBJECT_NAME_LENGTH,
+            strict_cors_mode=self.strict_cors_mode)
 
     def check_config(self):
         """
@@ -261,11 +264,6 @@ class Application(object):
         try:
             if self.memcache is None:
                 self.memcache = cache_from_env(env)
-            # Remove any x-backend-* headers since those are reserved for use
-            # by backends communicating with each other; no end user should be
-            # able to send those into the cluster.
-            for key in list(k for k in env if k.startswith('HTTP_X_BACKEND_')):
-                del env[key]
             req = self.update_request(Request(env))
             return self.handle_request(req)(env, start_response)
         except UnicodeError:
