@@ -22,6 +22,7 @@ from uuid import uuid4
 
 from swift.account.backend import AccountBroker
 from swift.common.utils import normalize_timestamp
+from swift.common.db import DatabaseConnectionError
 
 
 class TestAccountBroker(unittest.TestCase):
@@ -31,13 +32,18 @@ class TestAccountBroker(unittest.TestCase):
         # Test AccountBroker.__init__
         broker = AccountBroker(':memory:', account='a')
         self.assertEqual(broker.db_file, ':memory:')
-        got_exc = False
         try:
             with broker.get() as conn:
                 pass
-        except Exception:
-            got_exc = True
-        self.assert_(got_exc)
+        except DatabaseConnectionError as e:
+            self.assertTrue(hasattr(e, 'path'))
+            self.assertEquals(e.path, ':memory:')
+            self.assertTrue(hasattr(e, 'msg'))
+            self.assertEquals(e.msg, "DB doesn't exist")
+        except Exception as e:
+            self.fail("Unexpected exception raised: %r" % e)
+        else:
+            self.fail("Expected a DatabaseConnectionError exception")
         broker.initialize(normalize_timestamp('1'))
         with broker.get() as conn:
             curs = conn.cursor()
