@@ -24,7 +24,7 @@ from mock import patch
 from contextlib import nested
 
 from swift.account import reaper
-from swift.account.server import DATADIR
+from swift.account.backend import DATADIR
 from swift.common.exceptions import ClientException
 from swift.common.utils import normalize_timestamp
 
@@ -67,7 +67,7 @@ class FakeBroker(object):
         return self.info
 
 
-class FakeAccountBroker():
+class FakeAccountBroker(object):
     def __init__(self, containers):
         self.containers = containers
 
@@ -87,7 +87,7 @@ class FakeAccountBroker():
         return False
 
 
-class FakeRing():
+class FakeRing(object):
     def __init__(self):
         self.nodes = [{'id': '1',
                        'ip': '10.10.10.1',
