@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2011 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -49,7 +49,7 @@ class TestACL(unittest.TestCase):
         self.assertEquals(value, '.r:*')
         value = acl.clean_acl('header', '.referrer:*')
         self.assertEquals(value, '.r:*')
-        value = acl.clean_acl('header', 
+        value = acl.clean_acl('header',
                               ' .r : one , ,, .r:two , .r : - three ')
         self.assertEquals(value, '.r:one,.r:two,.r:-three')
         self.assertRaises(ValueError, acl.clean_acl, 'header', '.unknown:test')
@@ -81,6 +81,83 @@ class TestACL(unittest.TestCase):
             (['ref3', 'ref5', '-ref6'],
              ['acc1', 'acc2:usr2', 'acc3', 'acc4:usr4']))
 
+    def test_parse_v2_acl(self):
+        # For all these tests, the header name will be "hdr".
+        tests = [
+            # Simple case: all ACL data in one header line
+            ({'hdr': '{"a":1,"b":"foo"}'}, {'a': 1, 'b': 'foo'}),
+
+            # No header "hdr" exists -- should return None
+            ({}, None),
+            ({'junk': 'junk'}, None),
+
+            # Empty ACLs should return empty dict
+            ({'hdr': ''}, {}),
+            ({'hdr': '{}'}, {}),
+            ({'hdr': '{ }'}, {}),
+
+            # Bad input -- should return None
+            ({'hdr': '["array"]'}, None),
+            ({'hdr': 'null'}, None),
+            ({'hdr': '"some_string"'}, None),
+            ({'hdr': '123'}, None),
+        ]
+
+        for hdrs_in, expected in tests:
+            result = acl.parse_acl(version=2, data=hdrs_in.get('hdr'))
+            self.assertEquals(expected, result,
+                              '%r: %r != %r' % (hdrs_in, result, expected))
+
+    def test_format_v1_acl(self):
+        tests = [
+            ((['a', 'b'], ['c.com']), 'a,b,.r:c.com'),
+            ((['a', 'b'], ['c.com', '-x.c.com']), 'a,b,.r:c.com,.r:-x.c.com'),
+            ((['a', 'b'], None), 'a,b'),
+            ((None, ['c.com']), '.r:c.com'),
+            ((None, None), ''),
+        ]
+
+        for (groups, refs), expected in tests:
+            result = acl.format_acl(
+                version=1, groups=groups, referrers=refs, header_name='hdr')
+            self.assertEquals(expected, result, 'groups=%r, refs=%r: %r != %r'
+                              % (groups, refs, result, expected))
+
+    def test_format_v2_acl(self):
+        tests = [
+            ({}, '{}'),
+            ({'foo': 'bar'}, '{"foo":"bar"}'),
+            ({'groups': ['a', 'b'], 'referrers': ['c.com', '-x.c.com']},
+             '{"groups":["a","b"],"referrers":["c.com","-x.c.com"]}'),
+        ]
+
+        for data, expected in tests:
+            result = acl.format_acl(version=2, acl_dict=data)
+            self.assertEquals(expected, result,
+                              'data=%r: %r *!=* %r' % (data, result, expected))
+
+    def test_acls_from_account_info(self):
+        test_data = [
+            ({}, None),
+            ({'sysmeta': {}}, None),
+            ({'sysmeta':
+              {'core-access-control': '{"VERSION":1,"admin":["a","b"]}'}},
+             {'admin': ['a', 'b'], 'read-write': [], 'read-only': []}),
+            ({
+                'some-key': 'some-value',
+                'other-key': 'other-value',
+                'sysmeta': {
+                    'core-access-control': '{"VERSION":1,"admin":["a","b"],"r'
+                                           'ead-write":["c"],"read-only":[]}',
+                }},
+             {'admin': ['a', 'b'], 'read-write': ['c'], 'read-only': []}),
+        ]
+
+        for args, expected in test_data:
+            result = acl.acls_from_account_info(args)
+            self.assertEqual(expected, result, "%r: Got %r, expected %r" %
+                             (args, result, expected))
+
     def test_referrer_allowed(self):
         self.assert_(not acl.referrer_allowed('host', None))
         self.assert_(not acl.referrer_allowed('host', []))
@@ -105,13 +182,17 @@ class TestACL(unittest.TestCase):
             'http://user:pass@www.example.com:8080', ['.example.com']))
         self.assert_(acl.referrer_allowed('http://www.example.com',
                                           ['.example.com']))
-        self.assert_(not acl.referrer_allowed('http://thief.example.com',
+        self.assert_(not acl.referrer_allowed(
+            'http://thief.example.com',
             ['.example.com', '-thief.example.com']))
-        self.assert_(not acl.referrer_allowed('http://thief.example.com',
+        self.assert_(not acl.referrer_allowed(
+            'http://thief.example.com',
             ['*', '-thief.example.com']))
-        self.assert_(acl.referrer_allowed('http://www.example.com',
+        self.assert_(acl.referrer_allowed(
+            'http://www.example.com',
             ['.other.com', 'www.example.com']))
-        self.assert_(acl.referrer_allowed('http://www.example.com',
+        self.assert_(acl.referrer_allowed(
+            'http://www.example.com',
             ['-.example.com', 'www.example.com']))
         # This is considered a relative uri to the request uri, a mode not
         # currently supported.
