@@ -1,4 +1,4 @@
-# Copyright (c) 2012 OpenStack, LLC.
+# Copyright (c) 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,6 +18,7 @@ import unittest
 from swift.common.middleware import keystoneauth
 from swift.common.swob import Request, Response
 from swift.common.http import HTTP_FORBIDDEN
+from test.unit import FakeLogger
 
 
 class FakeApp(object):
@@ -36,12 +37,13 @@ class FakeApp(object):
                 return resp(env, start_response)
         status, headers, body = self.status_headers_body_iter.next()
         return Response(status=status, headers=headers,
-                              body=body)(env, start_response)
+                        body=body)(env, start_response)
 
 
 class SwiftAuth(unittest.TestCase):
     def setUp(self):
         self.test_auth = keystoneauth.filter_factory({})(FakeApp())
+        self.test_auth.logger = FakeLogger()
 
     def _make_request(self, path=None, headers=None, **kwargs):
         if not path:
@@ -163,10 +165,18 @@ class SwiftAuth(unittest.TestCase):
         resp = req.get_response(self._get_successful_middleware())
         self.assertEqual(resp.status_int, 200)
 
+    def test_auth_scheme(self):
+        req = self._make_request(path='/v1/BLAH_foo/c/o',
+                                 headers={'X_IDENTITY_STATUS': 'Invalid'})
+        resp = req.get_response(self.test_auth)
+        self.assertEqual(resp.status_int, 401)
+        self.assertTrue('Www-Authenticate' in resp.headers)
+
 
 class TestAuthorize(unittest.TestCase):
     def setUp(self):
         self.test_auth = keystoneauth.filter_factory({})(FakeApp())
+        self.test_auth.logger = FakeLogger()
 
     def _make_request(self, path, **kwargs):
         return Request.blank(path, **kwargs)
@@ -174,7 +184,8 @@ class TestAuthorize(unittest.TestCase):
     def _get_account(self, identity=None):
         if not identity:
             identity = self._get_identity()
-        return self.test_auth._get_account_for_tenant(identity['HTTP_X_TENANT_ID'])
+        return self.test_auth._get_account_for_tenant(
+            identity['HTTP_X_TENANT_ID'])
 
     def _get_identity(self, tenant_id='tenant_id', tenant_name='tenant_name',
                       user_id='user_id', user_name='user_name', roles=[]):
@@ -244,7 +255,8 @@ class TestAuthorize(unittest.TestCase):
         self.assertTrue(req.environ.get('swift_owner'))
 
     def _check_authorize_for_tenant_owner_match(self, exception=None):
-        identity = self._get_identity(user_name='same_name', tenant_name='same_name')
+        identity = self._get_identity(user_name='same_name',
+                                      tenant_name='same_name')
         req = self._check_authenticate(identity=identity, exception=exception)
         expected = bool(exception is None)
         self.assertEqual(bool(req.environ.get('swift_owner')), expected)
@@ -314,30 +326,55 @@ class TestAuthorize(unittest.TestCase):
             self._check_authenticate(identity=identity, acl=acl)
 
     def test_cross_tenant_authorization_success(self):
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantID:userA']), 'tenantID:userA')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantNAME:userA']), 'tenantNAME:userA')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['*:userA']), '*:userA')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME',
+                ['tenantID:userA']),
+            'tenantID:userA')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME',
+                ['tenantNAME:userA']),
+            'tenantNAME:userA')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME', ['*:userA']),
+            '*:userA')
 
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantID:userID']), 'tenantID:userID')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantNAME:userID']), 'tenantNAME:userID')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['*:userID']), '*:userID')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME',
+                ['tenantID:userID']),
+            'tenantID:userID')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME',
+                ['tenantNAME:userID']),
+            'tenantNAME:userID')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME', ['*:userID']),
+            '*:userID')
 
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantID:*']), 'tenantID:*')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantNAME:*']), 'tenantNAME:*')
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['*:*']), '*:*')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME', ['tenantID:*']),
+            'tenantID:*')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME', ['tenantNAME:*']),
+            'tenantNAME:*')
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME', ['*:*']),
+            '*:*')
 
     def test_cross_tenant_authorization_failure(self):
-        self.assertEqual(self.test_auth._authorize_cross_tenant('userID',
-            'userA', 'tenantID', 'tenantNAME', ['tenantXYZ:userA']), None)
+        self.assertEqual(
+            self.test_auth._authorize_cross_tenant(
+                'userID', 'userA', 'tenantID', 'tenantNAME',
+                ['tenantXYZ:userA']),
+            None)
 
     def test_delete_own_account_not_allowed(self):
         roles = self.test_auth.operator_roles.split(',')
