@@ -106,6 +106,18 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEquals(res.status_int, 413)
 
+    def test_exceed_bytes_quota_copy_verb(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '2'}})
+
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                            'swift.object/a/c2/o2': {'length': 10},
+                            'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 413)
+
     def test_not_exceed_bytes_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
@@ -127,6 +139,17 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEquals(res.status_int, 200)
 
+    def test_not_exceed_bytes_quota_copy_verb(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                            'swift.object/a/c2/o2': {'length': 10},
+                            'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 200)
+
     def test_bytes_quota_copy_from_no_src(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
@@ -138,6 +161,27 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEquals(res.status_int, 200)
 
+    def test_bytes_quota_copy_from_bad_src(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c/o',
+                            environ={'REQUEST_METHOD': 'PUT',
+                            'swift.cache': cache},
+                            headers={'x-copy-from': 'bad_path'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 412)
+
+    def test_bytes_quota_copy_verb_no_src(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'bytes': 0, 'meta': {'quota-bytes': '100'}})
+        req = Request.blank('/v1/a/c2/o3',
+                            environ={'REQUEST_METHOD': 'COPY',
+                            'swift.object/a/c2/o2': {'length': 10},
+                            'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 200)
+
     def test_exceed_counts_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
@@ -159,6 +203,16 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEquals(res.status_int, 413)
 
+    def test_exceed_counts_quota_copy_verb(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '1'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                            'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 413)
+
     def test_not_exceed_counts_quota(self):
         app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
         cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
@@ -179,6 +233,16 @@ class TestContainerQuotas(unittest.TestCase):
         res = req.get_response(app)
         self.assertEquals(res.status_int, 200)
 
+    def test_not_exceed_counts_quota_copy_verb(self):
+        app = container_quotas.ContainerQuotaMiddleware(FakeApp(), {})
+        cache = FakeCache({'object_count': 1, 'meta': {'quota-count': '2'}})
+        req = Request.blank('/v1/a/c2/o2',
+                            environ={'REQUEST_METHOD': 'COPY',
+                            'swift.cache': cache},
+                            headers={'Destination': '/c/o'})
+        res = req.get_response(app)
+        self.assertEquals(res.status_int, 200)
+
     def test_invalid_quotas(self):
         req = Request.blank(
             '/v1/a/c',
