@@ -1,4 +1,4 @@
-# Copyright (c) 2010-2012 OpenStack, LLC.
+# Copyright (c) 2010-2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -25,8 +25,7 @@ from eventlet import spawn, Timeout, listen
 
 from swift.common import utils
 from swift.container import updater as container_updater
-from swift.container import server as container_server
-from swift.common.db import ContainerBroker
+from swift.container.backend import ContainerBroker, DATADIR
 from swift.common.ring import RingData
 from swift.common.utils import normalize_timestamp
 
@@ -41,11 +40,12 @@ class TestContainerUpdater(unittest.TestCase):
         os.mkdir(self.testdir)
         ring_file = os.path.join(self.testdir, 'account.ring.gz')
         with closing(GzipFile(ring_file, 'wb')) as f:
-            pickle.dump(RingData([[0, 1, 0, 1], [1, 0, 1, 0]],
-                [{'id': 0, 'ip': '127.0.0.1', 'port': 12345, 'device': 'sda1',
-                  'zone': 0},
-                 {'id': 1, 'ip': '127.0.0.1', 'port': 12345, 'device': 'sda1',
-                  'zone': 2}], 30),
+            pickle.dump(
+                RingData([[0, 1, 0, 1], [1, 0, 1, 0]],
+                         [{'id': 0, 'ip': '127.0.0.1', 'port': 12345,
+                           'device': 'sda1', 'zone': 0},
+                          {'id': 1, 'ip': '127.0.0.1', 'port': 12345,
+                           'device': 'sda1', 'zone': 2}], 30),
                 f)
         self.devices_dir = os.path.join(self.testdir, 'devices')
         os.mkdir(self.devices_dir)
@@ -63,7 +63,7 @@ class TestContainerUpdater(unittest.TestCase):
             'interval': '1',
             'concurrency': '2',
             'node_timeout': '5',
-            })
+        })
         self.assert_(hasattr(cu, 'logger'))
         self.assert_(cu.logger is not None)
         self.assertEquals(cu.devices, self.devices_dir)
@@ -81,9 +81,9 @@ class TestContainerUpdater(unittest.TestCase):
             'concurrency': '1',
             'node_timeout': '15',
             'account_suppression_time': 0
-            })
+        })
         cu.run_once()
-        containers_dir = os.path.join(self.sda1, container_server.DATADIR)
+        containers_dir = os.path.join(self.sda1, DATADIR)
         os.mkdir(containers_dir)
         cu.run_once()
         self.assert_(os.path.exists(containers_dir))
@@ -128,18 +128,20 @@ class TestContainerUpdater(unittest.TestCase):
                     self.assert_('x-delete-timestamp' in headers)
                     self.assert_('x-object-count' in headers)
                     self.assert_('x-bytes-used' in headers)
-            except BaseException, err:
+            except BaseException as err:
                 import traceback
                 traceback.print_exc()
                 return err
             return None
         bindsock = listen(('127.0.0.1', 0))
+
         def spawn_accepts():
             events = []
             for _junk in xrange(2):
                 sock, addr = bindsock.accept()
                 events.append(spawn(accept, sock, addr, 201))
             return events
+
         spawned = spawn(spawn_accepts)
         for dev in cu.get_account_ring().devs:
             if dev is not None:
@@ -163,8 +165,8 @@ class TestContainerUpdater(unittest.TestCase):
             'interval': '1',
             'concurrency': '1',
             'node_timeout': '15',
-            })
-        containers_dir = os.path.join(self.sda1, container_server.DATADIR)
+        })
+        containers_dir = os.path.join(self.sda1, DATADIR)
         os.mkdir(containers_dir)
         subdir = os.path.join(containers_dir, 'subdir')
         os.mkdir(subdir)
@@ -173,6 +175,7 @@ class TestContainerUpdater(unittest.TestCase):
         cb.initialize(normalize_timestamp(1))
         cb.put_object('\xce\xa9', normalize_timestamp(2), 3, 'text/plain',
                       '68b329da9893e34099c7d8ad5cb9c940')
+
         def accept(sock, addr):
             try:
                 with Timeout(3):
@@ -181,12 +184,14 @@ class TestContainerUpdater(unittest.TestCase):
                     out.write('HTTP/1.1 201 OK\r\nContent-Length: 0\r\n\r\n')
                     out.flush()
                     inc.read()
-            except BaseException, err:
+            except BaseException as err:
                 import traceback
                 traceback.print_exc()
                 return err
             return None
+
         bindsock = listen(('127.0.0.1', 0))
+
         def spawn_accepts():
             events = []
             for _junk in xrange(2):
@@ -194,6 +199,7 @@ class TestContainerUpdater(unittest.TestCase):
                     sock, addr = bindsock.accept()
                     events.append(spawn(accept, sock, addr))
             return events
+
         spawned = spawn(spawn_accepts)
         for dev in cu.get_account_ring().devs:
             if dev is not None:
